//
// Board specifc definitions for
// JUMA-TRX2 transceiver
// Juha Niinikoski OH2NLT 22.11.2006
//
// MPLAB C30 version
// 
// S-meter default calibration changed for Rev.B board. 2.0V = full scale. 17.03.2007
// Voice memory Rev.B board def changes 21.03.2007
// Button switch debounce time shortened 03.04.2007
// All Band PA filter select 15.04.2007
// PTT_IN changed to PTT_IN_IO for main logic change 02.06.2007
// CW TX side tone SCAF setup added 25.01.2008
// K250 & K500 filter switch points changed 18.04.2009
// DEFAULT_TX filter changed to 2,6kHz (2586Hz), 19.04.2009

// processor header
#include <p30f6014A.h>

// uart setup according clock config
#define BAUD 9600
//#define SERIAL_MODE 3					// test interface
#define SERIAL_MODE 1					// Yaesu CAT emulation


// clock constants

#define FCY  7500000UL			// 30MHz ext Osc / 4  = 7,5MHz

#define DLYCONST ((FCY/1000000UL) + 1)	// constant for timer delay routines


// TRX-2 common definitions, used by various modules

// DDS constants
#define REF_OSC 180000000				// 180,000 MHz ref osc nominal frequency
#define MAX_DDS_NORMAL  1431655765		// upper limit 60MHz, Nyquist frequency = 90MHz
#define MAX_DDS_SERVICE 1908874354		// upper limit 80MHz, Nyquist frequency = 90MHz
#define DEFAULT_FREQ_A 176523156		// "factory default" VFO A 3.699 MHz
#define DEFAULT_FREQ_B 336916371		// "factory default" VFO B 7.060 MHz
#define MIN_DDS 0						// low limit
#define M7_FILT 190887435				// 3,5 / 7MHz (two band PA) filter switch at 4MHz

// all band PA filter & attenuator switch points
//#define K250	11930465				// 250kHz 
//#define K500	23860929				// 500kHz

#define K250	14316558				// 300kHz, 18.04.2009 
#define K500	28633115				// 600kHz

#define M1		47721859
#define M2		95443718
#define M4		190887435
#define M8		381774871
#define M12		572662306
#define M15		715827883
#define M19		906715318
#define M23		1097602753
#define M26		1240768330

#define CW_SHIFT 33406					// default CW shift. 700 Hz down when CW TX
#define CW_SIDETONE 5357				// default tone 700 Hz
#define MIN_CW_SHIFT 14317				// 300 Hz
#define MAX_CW_SHIFT 52495				// 1100Hz

// RIT addjustment
#define RIT_SCALE 24L					// rit pot scaling factor
#define RIT_DEAD_ZONE 50				// rit pot center dead zone, A/D steps

// CW keyer
#define KEYER 2							// factory default = Iambic B

// LCD parameters & factory defaults
#define DEFAULT_CONTRAST 2000
#define MAX_CONTRAST 3500
#define DEFAULT_BL 100
#define MAX_BL 1100
#define DEFAULT_BLT 0					// LCD back light timer not in use
#define MAX_BLT 3600

// panel buttons
//#define BUTTON_DEBOUNCE 200		// debounce delay(ms) "original"
#define BUTTON_DEBOUNCE 100		// debounce delay(ms) for fast fingers(oh7sv)
#define LONG_PUSH 1000			// long button push
#define VERY_LONG_PUSH	3000	// very long push
#define POWER_OFF 1200			// power off push
#define DEFAULT_BEEP_LEN 50;	// button beep length 50ms, long = * 10

#define SQ_HOLD 500				// squelch hold time ms

// ADC values

// channell definitions
#define ID_CUR 9				// PA Drain current chanell
#define BATT_CH 10				// battery chanell
#define S_METER	11				// S-meter
#define REV_PWR 12				// reverse power chanell
#define FWD_PWR 13				// forward power
#define CW_POT 14				// CW speed pot, R24
#define RIT_POT 15				// RIT adjust pot, R9


// Tone generator constants, 7,5MHz / tone(Hz) * 2
// some fixed tones
#define HZ350	10714
#define HZ400	9375
#define HZ500	7500
#define HZ700	5357			// 700 Hz
#define HZ1000	3750
#define HZ1500	2500
#define HZ2000	1875

// some harmony sounds
#define HZ392_01	9566		// G, off
#define HZ466_85	8044		// Bp, push button standard tone
#define HZ587_31	6385		// D, fast tune tone
#define HZ698_45	5369		// F, CW side tone

// meter scaling factors, factory defaults
#define BATT_MULT 132UL			// 100 x battery voltage ADC * BATT_MULT / 256 
#define ID_MULT 6613UL			// default value for Drain current scaling, ID*100 = ADC * ID_MULT / 65536 
//#define	S_MTR_MULT 2134UL		// default value for S-meter scaling, S-meter = ADC * S_MTR_MULT / 65536, 48 = max meter =1.6V FS
#define	S_MTR_MULT 1920UL		// default value for S-meter scaling, S-meter = ADC * S_MTR_MULT / 65536, 48 = max meter = 2.0V FS
#define	REV_PWR_MULT 23UL		// default value for Reverse Power scaling
#define	FWD_PWR_MULT 23UL		// default value for Forward Power scaling,P*100 =  ADC^2 * FWD_PWR_MULT / 65536, 10W = 1000
#define PWR_GRAPH_MULT 18UL		// power bar graph,48=max = 10W, ADC^2 * PWR_GRAPH_MULT / 1048576
#define F_VOLT_GRAPH_MULT 1874UL // power bar graph,48=max = 10W, ADC * F_VOLT_GRAPH_MULT / 65536

// alarm limits
#define SWR_ALARM 300			// SWR alarm 3.00
#define SWR_AL_DLY 200			// PTT to alarm active delay about 400ms @ 2ms main loop

// SCAF filter clock
// divider = 7500000 / Filt(Hz)*100
#define DEFAULT_NAR	75			// 1kHz
//#define DEFAULT_MID	37			// 2,0kHz
#define DEFAULT_MID	34			// 2,2kHz
//#define DEFAULT_WID	28			// 2,7kHz
//#define DEFAULT_WID	29			// 2,6kHz
#define DEFAULT_WID	30			// 2,5kHz
//#define DEFAULT_TX	28			// 2,7kHz (2678Hz)
#define DEFAULT_TX	29			// 2,6kHz (2586Hz), 19.04.2009

#define SIDE_TONE_SCAF 50		// side tone filter for CW TX
#define MIN_SCAF	18			// set value low limit, 4000Hz
#define MID_SCAF	33			// set value wide-mid limit, 2300Hz
#define NAR_SCAF	58			// set value mid-nar limit, 1300Hz
//#define MAX_SCAF	111			// set value high limit, 675Hz
#define MAX_SCAF	150			// set value high limit, 500Hz

// Power on & calibration value addresses in the EEPROM
#define EEPAGE 0x7F				// EEPROM address high part
#define EEDEF 0xF000			// EEPROM default values storage area
#define EECAL 0xF100			// EEPROM calibration values storage area
#define EE_FD_LOC 0xF1FE		// counter address in EEPROM

// MIC / Line input
#define MIC 1					// Microphone input level / bias current on
#define LINE 0					// Line input level / bias off 

// TRX2 board I/O definitions

// Port A Switches

#define INIT_TRISA 0xFF3F				// PortA switch inputs
#define INIT_PORTA 0x0000

#define SW2	_RA12						// SET
#define SW1	_RA13						// FUNC
#define ENQ_B 	_RA14					// rotary encoder, phase B / INT 3
#define ENQ_A 	_RA15					// rotary encoder, phase A / INT 4


// Port B ADC inputs

#define INIT_TRISB 0xFE73				// PortB ADC and IDC pins = inputs
#define INIT_PORTB 0x0000

#define PTT_IN_IO	_RB6				// PTT input / TX on, 0 = active
#define PTT_OUT	_LATB7					// PTT output, 1 = active

#define DOT		_RB5					// paddle DOT input, 0 = active
#define DASH	_RB4					// paddle DASH input, 0 = active
#define KEY		_LATB3					// key output, 1 = active

// Port C Switches & power control

#define INIT_TRISC 0x401E				//
#define INIT_PORTC 0x0000				//

#define SW3		_RC4					// RIT		, 0 = switch pushed
#define SW4		_RC3					// VFO
#define SW5		_RC2					// MODE
#define SW6		_RC1					// FILTER

#define PWR_SW	_RC14					// Power switch input, 1 = switch pushed
#define PWR_ON	_RC13					// Power on


// Port D LCD & PWM out

#define INIT_TRISD 0x0000			// LCD & LED I/O, all outputs
#define INIT_PORTD 0x0000			//

// tone output
#define TONE_OUT	_LATD1			// tone output
#define TONE_TRIS	_TRISD1			// tone output tri state control

// LCD signals
#define LCD_E		_LATD5			// LCD E signal

#define LCD_RW		_LATD6			// LCD R/W signal
#define LCD_RS		_LATD7			// LCD RS signal

#define LCD_DATA	LATD			// LCD 8-bit data bus out register
#define LCD_BUS		PORTD			// LCD 8-bit data bus state
#define LCD_TRIS	TRISD			// LCD data direction 

#define LCD_SHIFT 8					// LCD bits shifted from D0
#define LCD_MASK	(0x00FF<<LCD_SHIFT)			// LCD data lines at port


// PORT F & G 
// RS232, DDS, SPI and Codec/AUX signals

//#define INIT_TRISF 0xFC7C			//
#define INIT_TRISF 0xFE18
#define INIT_PORTF 0x0000

#define INIT_TRISG 0x1080			// I/O directions for ISD17240 add on board
#define INIT_PORTG 0x0000

#define DDS_SER		_LATF0			// AD9850 DDS D7/SER
#define DDS_RESET	_LATG0			// AD9850 DDS Reset
#define DDS_FQ_UD	_LATG1			// AD9850 DDS FQ_UD
#define DDS_W_CLK	_LATF1			// AD9850 DDS W_CLK	


// 74HC595 Main board shift register bits
// shift register is loaded with SPI1 functions, MSB goes first

#define SSB_CW		(1 << 0)		// SSB / !CW select
#define SB_SELECT	(1 << 1)		// sideband select, 0 = ??, 1 = ??
#define NAR			(1 << 2)		// Narrow BPF filter, 1 = Narrow
#define FAST_AGC	(1 << 3)		// Fast AGC select, 1 = Fast
#define PROC_ON		(1 << 4)		// TX speech processor on, 1 = on
#define NB			(1 << 5)		// Noice blacer on, tbd
#define	AF_INPUT	(1 << 6)		// Microphone / Line input select, 0=Line, 1=MIC
#define MHZ7		(1 << 7)		// 3,5 / 7 MHz filter change


// ISD17240 Voice Recorder I/O
//
#define ISD_17240	0xE0			//ISD17240 Device ID

#define ISD_PLAY_BAND _LATG15		//out Play to Band switch, 1=play band

#define ISD_RESET	_LATG14			//out ISD17240 nRESET
#define ISD_RDY		_RG12			//in  ISD17240 Ready / INT

#define ISD_SS		_LATG9			//out ISD17240 SPI nSS
#define ISD_SCLK	_LATG6			//out ISD17240 SPI SCLK
#define ISD_MOSI	_LATG8			//out ISD17240 SPI MOSI
#define ISD_MISO	_RG7			//in  ISD17240 SPI MISO

