//
// Simple UART2 interface for JUMA-TRX2
// Juha Niinikoski, OH2NLT 01.02.2008
//
// data type cast corrections 18.11.2008

#include "juma-trx2.h"
#include <stdio.h>

// Baud Rate divisors for 30 MHz osc =  7,5 MHz FCY 
/* actual baud rate divisors
[0] 1200	389,625
[1] 2400	194,3125
[2] 4800	96,65625
[3] 9600	47,828125
[4] 19200	23,4140625
[5] 38400	11,20703125
[6] 57600	7,138020833
[7] 115200	3,069010417
*/

/* Module variables & definitions */

static volatile unsigned char rx_buffer2[256];      // UART mod(256) RX queue, must be exactly 256 long
static volatile unsigned char rx_buf_out_idx2 = 0;  // buffer output index, pointer behind in idx
static volatile unsigned char rx_buf_in_idx2 = 1;   // buffer input index, pointer always "leading"

const unsigned int u2_baud_rates[8] = {390, 194, 97, 48, 23, 11, 7, 3};


// UART2 IRQ service
// queue character
void __attribute__((interrupt)) _U2RXInterrupt(void)  	// put receiced character to RX queue
	{
  	unsigned char rx;

  	rx = U2RXREG;                        				// data read
  	if((unsigned char)(rx_buf_in_idx2 + 1) != rx_buf_out_idx2) 		// check overrun
  		{
 		rx_buffer2[rx_buf_in_idx2] = rx;    		   // place just received character in queue
		rx_buf_in_idx2++;
    	}
  	// else reject character
// check & handle possible errors
	if(U2STAbits.OERR == 1)								// OERR is blocking uart, should not happen but..
		U2STAbits.OERR = 0;								// reset overrun error if occured

// this is not automatic in 30F6014 UART
	IFS1bits.U2RXIF = 0;								// reset irq flag
	}


// UART2 init
void InitUSART2(void)
	{
	//UBRG = ((FCY/16)/BAUD) - 1;   		// set baud rate
	U2BRG = u2_baud_rates[3];				// set intial baud rate to 9600
	U2MODE = 0x0000;
	U2STA = 0x0400;							// IRQ on every character, enable transmitter
 	IFS1bits.U2RXIF = 0;					// clear possible pending irq
 	IEC1bits.U2RXIE = 1;					// enable RX IRQs
 	U2MODEbits.UARTEN = 1;					// enable uart
	U2STAbits.UTXEN = 1;					// enable transmitter, UART2 needs this after uart enable !
	}


// set UART2 baud rate
void SetUSART2baud(int baud_sel)
	{
	if(baud_sel > 7)
		baud_sel = 7;						// stay in the table

	 U2BRG = u2_baud_rates[baud_sel];		// set baud rate
	}


// UART2 print character
void putch2(char c)
	{
	while(U2STAbits.UTXBF == 1)				// wait here if tx fifo full
		{
		}
	U2TXREG = (int)c &0x00FF;				// Tx reg is 16-bit long
	}


// UART2 print null terminated string
void putst2(char *str)
	{
	while((*str)!=0)
		{
		putch2(*str);
//	   	if (*str==13) putch2(10);
//    	if (*str==10) putch2(13);
		str++;
		}
	}



// check if key perssed, data available from receive buffer
// Not pressed = 0, Pressed = number of characters
int kbhit2(void)
	{
	unsigned char ret;

	ret = rx_buf_in_idx2  - (unsigned char)(rx_buf_out_idx2 + 1); // return with number of available characters
	return(ret);
	}


// clear serial port, reset input buffer
void ClearUSART2queue(void)
	{
	rx_buf_out_idx2 = 0;  								// buffer output index
	rx_buf_in_idx2 = 1;   								// buffer input index
	}


// get char from RX queue, IRQ RX system
unsigned char getch2(void)
	{
 	while((unsigned char)(rx_buf_out_idx2 + 1) == rx_buf_in_idx2);   	// wait data if queue is empty 
  	return rx_buffer2[++rx_buf_out_idx2];             	// return with data
	}


// receive character with echo
unsigned char getche2(void)
	{
	unsigned char c;
	putch2(c = getch2());
	return c;
	}
