//
// JUMA-TRX2 UART2 external serial keyboard for Voice Memory control
//
// Juha Niinikoski, OH2NLT 02.01.2008
//
// UART2 connected to JUMA-TRX2 AUX connector can be used for 
// Voice Memory control. UART1 is left for other tasks
//

#include "juma-trx2.h"				// board definitions
#include "lcd-trx2.h"				// LCD specific definitions

// EEPROM structures
#include "trx2_eeprom.h"			// get EEPROM structure definitions


// external references
// external functions

// system calibration values

extern union
	{
	struct calval calval;
	unsigned int ee[sizeof(struct calval)/2];
	}cal;


// UART 2 I/O
extern int kbhit2(void);
extern unsigned char getch2(void);


// ISD1700 test
extern void init_spi2(void);
extern unsigned int tx_rx_16_spi2(unsigned int dta);
extern unsigned int tx_rx_32_spi2(unsigned int cmd1, unsigned int dta2 );
extern unsigned int tx_rx_56_spi2(unsigned int cmd, unsigned int dta1, unsigned int dta2 );
extern void set_isd_addresses(int idx);
extern void play_mem(int idx, int mode);
extern void rec_mem(int idx, int ch);

// sound effect
extern void beep(int, int);												// generate beep, period, length


// external data
extern int isd_1700_installed;											// voice memory board found


// module variables
static int voice_mem;													// selected voice memory bank, 0-9
static unsigned char cmd;												// stored command


// Type 2 interface, this is in use
// Select function first and then start with memory number select


void uart2_ext_kb(void)
	{
	int x;
	unsigned char c;

		if(kbhit2())
			{
			c = getch2();

			if(isd_1700_installed == 0)
				return;													// do nothing if board is not installed

			switch(c)
				{
// store valid commands

				case 'P': case 'T': case 'R': case 'M':
					cmd = c;
					beep(HZ587_31, cal.calval.beep_len);				// short beep for push noticed
					break;

// service & test commands, excecute immediatelly

				case 'S':
					tx_rx_16_spi2(0x0002 | 0x0010);
					beep(HZ587_31, cal.calval.beep_len);				// short beep for push noticed
					cmd = 0;											// void command
					break;

				case 'E':
					beep(HZ587_31, cal.calval.beep_len*10);				// long beep for push noticed
					tx_rx_16_spi2(0x0043 | 0x0010);
					cmd = 0;						// void command
					break;

// select memory bank
// 0 = long memory for recording from RX
// others are short message memories
				case '0' :case '1': case '2': case '3': case '4': case '5': case '6': case '7': case '8': case '9':
					voice_mem = c-'0';
					set_isd_addresses(voice_mem);

// start stored command

				switch(cmd)
					{
// ISD1700 commands

// play to speaker
					case 'P':
						play_mem(voice_mem, 0);
						break;

// TX on the air
					case 'T':
						ISD_PLAY_BAND = 1;					// connect ISD1700 to mic ch
						PTT_OUT = 1;						// TX on
						play_mem(voice_mem, 1);				// Play message, stop at EOM or end address
						PTT_OUT = 0;						// TX off
						ISD_PLAY_BAND = 0;					// disconnect ISD1700 from MIC ch
						break;

// Record from RX audio
					case 'R':
						rec_mem(voice_mem, 1);
						break;

// Record from microphone
					case 'M':
						rec_mem(voice_mem, 0);
						break;
					}					// end switch(cmd)
					break;

				}						// end switch(c)
			}							// end if
	}									// end function

